@@ -11,13 +11,13 @@ fn main(m: Mailbox<()>) {
     let module = env.add_this_module().unwrap();
 
     // This process will fail because it can't uses syscalls for std i/o
-    let (_, m) = module
+    let (_, _, m) = module
         .spawn_link(m, |_: Mailbox<()>| println!("Hi from different env"))
         .unwrap();
     assert!(m.receive().is_signal());
 
     // This process will fail because it uses too much memory
-    let (_, m) = module
+    let (_, _, m) = module
         .spawn_link(m, |_: Mailbox<()>| {
             vec![0; 150_000];
         })
@@ -25,6 +25,6 @@ fn main(m: Mailbox<()>) {
     assert!(m.receive().is_signal());
 
     // This process will fail because it uses too much compute
-    let (_, m) = module.spawn_link(m, |_: Mailbox<()>| loop {}).unwrap();
+    let (_, _, m) = module.spawn_link(m, |_: Mailbox<()>| loop {}).unwrap();
     assert!(m.receive().is_signal());
 }
