@@ -1,6 +1,8 @@
+use std::{process::exit, time::Duration};
+
 use lunatic::{
     process::{self, Process},
-    Mailbox, Request,
+    Mailbox, ReceiveError, Request,
 };
 
 #[lunatic::test]
@@ -60,7 +62,7 @@ fn message_resource(m: Mailbox<Proc>) {
     let _child = process::spawn_with(this, |parent, _: Mailbox<()>| {
         let empty_proc = process::spawn(|_: Mailbox<i32>| {}).unwrap();
         parent.send(Proc(empty_proc));
-        let panic_proc = process::spawn(|_: Mailbox<i32>| panic!()).unwrap();
+        let panic_proc = process::spawn(|_: Mailbox<i32>| exit(1)).unwrap();
         parent.send(Proc(panic_proc));
     })
     .unwrap();
@@ -95,6 +97,15 @@ fn request_reply(m: Mailbox<u64>) {
     }
 }
 
+#[lunatic::test]
+fn timeout(m: Mailbox<u64>) {
+    let result = m.receive_timeout(Duration::new(0, 1000));
+    match result {
+        Err(ReceiveError::Timeout) => (), // success
+        _ => unreachable!(),
+    };
+}
+
 #[derive(serde::Serialize, serde::Deserialize)]
 struct Proc(Process<i32>);
 
